/*
 * ******************************************************************************************
 *
 * File		: apb_uart_dev.h
 * Author	: GOWIN
 * Function	: Device definition of APB UART
 *
 * ******************************************************************************************
 */

#ifndef __APB_UART_DEV_H__
#define __APB_UART_DEV_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include "apb_uart.h"


// Definitions ------------------------------------------------------------------------------

/*************************************************************************************
 * An APB UART device "APB UART1" is defined as a user template.
 * User can define more APB UART devices.
 *************************************************************************************/

/*************************************************************************************
 * "X" represents user customized.
 *************************************************************************************/

#define APB_UART1_BASE					APB_EXT_BASE/*X*/						// APB UART1 address definition
#define APB_UART1						((APB_UART_RegDef *) APB_UART1_BASE )	// APB UART1 register definition


#define IRQ_APB_UART1_SOURCE			IRQ_GP0_SOURCE/*X*/						// APB UART1 interrupt source
#define apb_uart1_irq_handler			gp0_irq_handler/*X*/					// APB UART1 interrupt handler


// APB UART1 with DMA
#if (APB_UART_DMA_EN == 0)
#define APB_UART1_DMA_EN				0
#else
#define APB_UART1_DMA_EN             	1/*X*/		// Enable/disable APB UART1 DMA TX or RX. X: 1: enable; 0: disable.
#endif
// DMA TX
#define APB_UART1_DMA_TX_CH             0/*X*/		// DMA channel number for APB UART1 TX
#define APB_UART1_DMA_TX_REQID          0/*X*/		// DMA request ID for APB UART1 TX
// DMA RX
#define APB_UART1_DMA_RX_CH             1/*X*/		// DMA channel number for APB UART1 RX
#define APB_UART1_DMA_RX_REQID          1/*X*/		// DMA request ID for APB UART1 RX


// Trigger level definitions
// Can be defined in C processor by user.
#ifndef APB_UART1_TRIG_LVL
#define APB_UART1_TRIG_LVL            	APB_UART_TRIG_LVL_1
#endif

// APB UART1
static APB_UART_INFO apb_uart1_info = {0};

#if (APB_UART1_DMA_EN == 1)
// DMA TX
void apb_uart1_dma_tx_event (unsigned int event);
static APB_UART_DMA apb_uart1_dma_tx =
{
	APB_UART1_DMA_TX_CH,
	APB_UART1_DMA_TX_REQID,
	apb_uart1_dma_tx_event
};

// DMA RX
void apb_uart1_dma_rx_event (unsigned int event);
static APB_UART_DMA apb_uart1_dma_rx =
{
	APB_UART1_DMA_RX_CH,
	APB_UART1_DMA_RX_REQID,
	apb_uart1_dma_rx_event
};
#endif

// Resources
static const APB_UART_RESOURCES apb_uart1_resources =
{
	{
		// Capabilities
		1,  // Supports APB UART (Asynchronous) mode
		0,  // Supports Synchronous Master mode
		0,  // Supports Synchronous Slave mode
		0,  // Supports APB UART Single-wire mode
		0,  // Supports APB UART IrDA mode
		0,  // Supports APB UART Smart Card mode
		0,  // Smart Card Clock generator
		0,  // RTS Flow Control available
		0,  // CTS Flow Control available
		0,  // Transmit completed event: APB_UART_EVENT_TX_COMPLETE
#if ((APB_UART1_DMA_EN == 1) || (APB_UART1_TRIG_LVL == APB_UART_TRIG_LVL_1))
		0,  // Signal receive character timeout event: APB_UART_EVENT_RX_TIMEOUT
#else
		1,  // Signal receive character timeout event: APB_UART_EVENT_RX_TIMEOUT
#endif
		0,  // RTS Line: 0=not available, 1=available
		0,  // CTS Line: 0=not available, 1=available
		0,  // DTR Line: 0=not available, 1=available
		0,  // DSR Line: 0=not available, 1=available
		0,  // DCD Line: 0=not available, 1=available
		0,  // RI Line: 0=not available, 1=available
		0,  // Signal CTS change event: APB_UART_EVENT_CTS
		0,  // Signal DSR change event: APB_UART_EVENT_DSR
		0,  // Signal DCD change event: APB_UART_EVENT_DCD
		0,  // Signal RI change event: APB_UART_EVENT_RI
	},
	APB_UART1,
	IRQ_APB_UART1_SOURCE,
	APB_UART1_TRIG_LVL,
#if (APB_UART_DMA_EN == 1)
	&apb_uart1_dma_tx,
	&apb_uart1_dma_rx,
#endif
	&apb_uart1_info
};


// Functions
APB_UART_CAPABILITIES apb_uart1_get_capabilities (void);
int apb_uart1_initialize (APB_UART_SignalEvent_t cb_event);
int apb_uart1_uninitialize (void);
int apb_uart1_power_control (APB_UART_POWER_STATE state);
int apb_uart1_send (const void *data, unsigned int num);
int apb_uart1_receive (void *data, unsigned int num);
int apb_uart1_transfer (const void *data_out, void *data_in, unsigned int num);
unsigned int apb_uart1_get_txcount (void);
unsigned int apb_uart1_get_rxcount (void);
int apb_uart1_control (unsigned int control, unsigned int arg);
APB_UART_STATUS apb_uart1_get_status (void);
int apb_uart1_set_modem_control (APB_UART_MODEM_CONTROL control);
APB_UART_MODEM_STATUS apb_uart1_get_modem_status (void);
#if (APB_UART1_DMA_EN == 1)
void apb_uart1_dma_tx_event (unsigned int event);
void apb_uart1_dma_rx_event (unsigned int event);
#endif
void apb_uart1_irq_handler (void);


// As this template, user can define more APB UART devices.


#ifdef __cplusplus
}
#endif


#endif	/* __APB_UART_DEV_H__ */
